#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>

#include "strplus.h"
#include "strctype.h"

	char *
strestr(s1, s2)
	const char *s1;
	const char *s2;
{
	return strstr(s1,s2) == NULL ? NULL : strendstr(s1,s2);
}

#if ! defined(SELF) && ! defined(SELFTEST)

/*
 * Functions strtolower() and strtoupper(), used by function below,
 * are defined in strctype.c so when we are building self-testing
 * binary, skip this section.
 */

	char *
stristr(s1, s2)
	const char *s1;
	const char *s2;
{
	char *a_s1, *a_s2;
	register char *ret;

	a_s1 = strdup(s1);
	a_s2 = strdup(s2);

	if (a_s1 == NULL) {
		free(a_s1);
		return NULL;
	}

	if (a_s2 == NULL) {
		free(a_s2);
		return NULL;
	}

	if ((ret = strstr(strtolower(a_s1), strtoupper(a_s2))) != NULL)
		ret = (char *) s1 + (ret - a_s1);

	free(a_s1);
	free(a_s2);

	return ret;
}

#endif

	char *
str_white_str(str, substr, size)
	char *str;
	char *substr;
	int  *size;
{
#if 0
	/*
	 * This is fastfix code substitution for str_white_str() function,
	 * because new wersion from 'Crasher' was not fully tested yet.
	 */

	*size = strlen(substr);
	return strstr(str, substr);

#else
	register int slen, plen, ssize;
	register char *pptr, *sptr, *start;

	slen = strlen(str);
	plen = strlen(substr);

	for (start = str, pptr = substr; slen >= plen; start++, slen--) {

		/* Find start of pattern in string. */
		while (*start != *substr) {

			if ((isspace(*start)) && (isspace(*substr)))
				break;

			start++;
			slen--;
			/* If pattern longer than string. */
			if (slen < plen)
				return NULL;
		}

		ssize = 0;
		sptr = start;
		pptr = substr;

		while (1) {

#ifdef DEBUG_STRPLUS /* if str_white_str() works properly, delete this */
			printf("comparing %d [%s] with %d [%s]\n",
					*sptr, sptr, *pptr, pptr);
#endif

			if ((isspace(*sptr)) && (isspace(*pptr))) {
				while (isspace(*sptr)) {
					++sptr;

					if (isspace(*pptr)) {
						++ssize;
						++pptr;
					}
					else
						++ssize;
				}
			}
			else if (*sptr == *pptr) {
				while (*sptr == *pptr && *sptr != '\0' && ! isspace(*sptr)) {
					sptr++;
					pptr++;
					ssize++;
				}
			}
			else {
				break;
			}

			/* If end of pattern then pattern was found. */
			if (*pptr == '\0') {
				if (size != NULL)
					*size = ssize;
				return start;
			}
		}
	}

	return NULL;
#endif
}


	int
strcnt(str, c)
	const char *str;
	const int c;
{
	register int i = 0;

	if (str != NULL)
		while (*str != '\0')
			if (*str++ == (char) c)
				i++;

	return i;
}

	int
strcnt_str(str, substr)
	const char *str;
	const char *substr;
{
	register char *s;
	register int count;

	for (count = 0; ; count++) {
		if ((s = strstr(str, substr)) == NULL)
			break;
		else
			str += (s - str) + 1;
	}

	return count;
}

	int
strcnt_sepstr(str, substr)
	const char *str;
	const char *substr;
{
	register char *s;
	register int count, substr_size;

	substr_size = strlen(substr);

	for (count = 0; ; count++) {
		if ((s = strstr(str, substr)) == NULL)
			break;
		else
			str += (s - str) + substr_size;
	}

	return count;
}

	char *
strdel(s)
	char *s;
{
	register int i;

	/* XXX: memmove() will be probably faster here */
	for (i = 0; s[i] != '\0'; i++)
		s[i] = s[i + 1];

	return s;
}

	char *
strrmlf(s)
	char *s;
{
	register char *p_lf;

	while ((p_lf = strchr(s, '\n')) != NULL)
		strdel(p_lf);

	return s;
}

	char *
strrmcr(s)
	char *s;
{
	register char *p_cr;

	while ((p_cr = strchr(s, '\r')) != NULL)
		strdel(p_cr);

	return s;
}

	char *
str_left_trim(s)
	char *s;
{
	register char *pos;

	for (pos = s; *pos != '\0' && isspace(*pos); pos++) ;

	if (pos > s)
		memmove((void *) s, (void *) pos, strlen(pos) + 1);

	return s;
}

	char *
str_right_trim(s)
	char *s;
{
	register char *pos;

	for (pos = s + (strlen(s) - 1); pos >= s && isspace(*pos); pos--) ;

	*(pos + 1) = '\0';

	return s;
}

	char *
str_trim_whitechars(s)
	char *s;
{
	register char *pos, *start;

	for (pos = s, start = NULL; ; pos++) {
		if (isspace(*pos)) {
			if (start == NULL)
				start = pos;
		}
		else {
			if (start != NULL) {
				memmove(start + 1, pos, strlen(pos) + 1);
				*start = ' ';

				pos = start + 1;
				start = NULL;
			}
		}

		if (*pos == '\0')
			break;
	}

	return s;
}

	char *
strins(str, ins)
	char *str;
	char *ins;
{
	register int ins_len = strlen(ins);

	memmove(str + ins_len, str, strlen(str) + 1);
	strncpy(str, ins, ins_len);

	return str;
}

	int
strrcmp(s1, s2)
	const char *s1;
	const char *s2;
{
	register char *x1, *x2;

	x1 = strchr(s1,'\0');
	x2 = strchr(s2,'\0');

	while (x1 > s1 && x2 > s2) {
		x1--;
		x2--;
		if (strcmp(x1,x2))
			return strcmp(x1,x2);
	}

	return strlen(s1) - strlen(s2);
}

#if defined(SELF) || defined(SELFTEST) || defined(SELF_STRPLUS)

#include <stdio.h>

#define TESTSTR1 "___AAA_BBBB__C_DaDaD____"
#define TESTSEP1 '_'
#define TESTSEP2 "BB"

#define TESTSTR2 " \t  AAA\nBBBB__C   D\taDa D \t   \t"

void strins_selftest(void) {
	char long_str[80];

#define SEARCH_STR	"56"
#define INSERT_STR	"<-now-goes-56->"

	strcpy(long_str, "1234567890");
	printf("  long_str before: %s\n", long_str);

	printf("  Now we're searching \"%s\" and want to insert \"%s\" before it.\n",
			SEARCH_STR, INSERT_STR);

	strins(strstr(long_str, SEARCH_STR), INSERT_STR);
	printf("  long_str  after: %s\n", long_str);

	return;
}

	int
main(argc, argv)
	int argc;
	char **argv;
{
	char *str2;

	puts("Entering str_white_str() selftest:");

	if (argc > 2) {
		int size;

		str2 = str_white_str(argv[1], argv[2], &size);
		/* str2 = str_white_str("telnet \t atlantis.sk 	5678", "t  a", &size);
		 */
		printf("  ptr = [%s], size = %d\n", str2, size);

		return 0;
	}

	printf("  Usage: %s <string> <substring>\n\n", argv[0]);

	str2 = strdup(TESTSTR2);

	printf("strcnt(\"%s\", '%c') = %d\n",
			TESTSTR1, TESTSEP1,
			strcnt(TESTSTR1, TESTSEP1));

	printf("strcnt_str(\"%s\", \"%s\") = %d\n",
			TESTSTR1, TESTSEP2,
			strcnt_str(TESTSTR1, TESTSEP2));

	printf("strcnt_sepstr(\"%s\", \"%s\") = %d\n",
			TESTSTR1, TESTSEP2,
			strcnt_sepstr(TESTSTR1, TESTSEP2));

	printf("str_trim_whitechars(\"%s\") = \"%s\"\n",
			TESTSTR2, str_trim_whitechars(str2));

	free(str2);

	puts("\nEntering strins_selftest():");
	strins_selftest();

	return 0;
}

#endif

