/**************************************************************************
 *                                                                        *
 *  Special string functions and macros header file                       *
 *                                                                        *
 *  Programmed by Ondrej Jombik <nepto@pobox.sk>                          *
 *  (c) Copyright 1997-1999 Alfa & Omega software Limited and StrangeARTS *
 *  (c) Copyright 2000 Condy software inc.                                *
 *  (c) Copyright 2001-2002 Platon software development group             *
 *  All rights reserved.                                                  *
 *                                                                        *
 *  Updates: 16.4.2000, 8.11.2000, 5.10.2001, 21.10.2001, 5.12.2001       *
 *  20.12.2001 - str_white_str() added (thanks to <rbarlik@yahoo.com>)    *
 *   4. 2.2002 - strins() added                                           *
 *  28. 2.2002 - str_white_str() bugfix                                   *
 *                                                                        *
 **************************************************************************/

#ifndef _PLATON_STR_STRPLUS_H
#define _PLATON_STR_STRPLUS_H

/* TODO: macros should not be in extern "C" { } */

#ifdef __cplusplus
extern "C" {
#endif

/*
 * Locate functions
 *
 * strendstr() - macro that locates end of substring (use only if you are sure,
 *               that substring is located in string)
 * strestr() - locate a end of substring
 * strcasestr(), stristr() - locate a substring ignoring case
 * str_white_str() - search substr in str; every white char in substr can be
 *                   substitued with one or more white chars in str; in size,
 *                   if not NULL, will be the length of matched patern;
 *                   author: Rastislav 'Crasher' Barlik <rbarlik@yahoo.com>
 *                   patched by: Ondrej Jombik <nepto@pobox.sk> [28/2/2002]
 * strwstr() - the same as function above but without size parameter
 */

#define strendstr(__s1,__s2) (strstr(__s1,__s2) + strlen(__s2))

char *strestr(const char *s1, const char *s2);
char *stristr(const char *s1, const char *s2);

#define strcasestr(__s1, __s2) stristr(__s1, __s2)

char *str_white_str(char *str, char *substr, int *size);

#define strwhitestr(str, substr, size) str_white_str(str, substr, size)
#define strwstr(str, substr) str_white_str(str, substr, NULL)

/*
 * Counting functions
 *
 * strcnt() - count number of characters in string
 * strcnt_str() - count number of substrings in string
 * strcnt_sepstr() - count number of separate substrings in string
 */

int strcnt(const char *str, const int c);
int strcnt_str(const char *str, const char *substr);
int strcnt_sepstr(const char *str, const char *substr);

/*
 * Removing functions
 *
 * strdel() - deletes a one character at position of s
 * strrmlf() - removes all occurences of LF (Line Feed) in string
 * strrmcr() - removes all occurences of CR (Carriage Return) in string.
 * strrmeol(), strrmcrlf(), strrmlfcr() - macros that removes all occurences
 *                                        of CR and LF in string
 * str_left_trim(), ltrim() - removes white characters from beginning of string
 * str_right_trim(), rtrim() - removes white characters from end of string
 * str_trim(),strtrim(), trim() - removes white characters from beginning and
 *                                end of string
 * str_trim_whitechars() - substitute every group of white characters in string by
 *                         one space character
 */

char *strdel(char *s);

char *strrmlf(char *s);
char *strrmcr(char *s);

#define strrmeol(__s)	strrmcr(strrmlf(__s))
#define strrmcrlf(__s)	strrmeol(__s)
#define strrmlfcr(__s)	strrmeol(__s)

char *str_left_trim(char *s);
char *str_right_trim(char *s);
char *str_trim_whitechars(char *s);

#define ltrim(s)	str_left_trim(s)
#define rtrim(s)	str_right_trim(s)
#define trim(s)		rtrim(ltrim(s))
#define strtrim(s)	trim(s)
#define str_trim(s)	trim(s)

/*
 * Inserting functions
 *
 * strins() - function inserts string ins at position str;
 *            there MUST be enough memory allocated in str
 */
char *strins(char *str, char *ins);

/*
 * Comparing functions
 *
 * strrcmp() - compares two string in reverse order
 */

int strrcmp(const char *s1, const char *s2);

#ifdef __cplusplus
}
#endif

#endif /* #ifndef _PLATON_STR_STRPLUS_H */

